SystemOrganization addCategory: #'Constraints-Tests'!
SystemOrganization addCategory: #'Constraints-Core'!

!ContextPart methodsFor: '*Constraints' stamp: 'mg 4/22/2014 13:32'!
doPrimitive: primitiveIndex method: meth receiver: receiver args: arguments 
	"Simulate a primitive method whose index is primitiveIndex.  The
	 simulated receiver and arguments are given as arguments to this message.
	 If successful, push result and return resuming context, else ^ PrimitiveFailToken.
	 Any primitive which provokes execution needs to be intercepted and simulated
	 to avoid execution running away."
	<layer: #ccm>
	| value |
	"Judicious use of primitive 19 (a null primitive that doesn't do anything) prevents
	 the debugger from entering various run-away activities such as spawning a new
	 process, etc.  Injudicious use results in the debugger not being able to debug
	 interesting code, such as the debugger itself.  hence use primtiive 19 with care :-)"
	"SystemNavigation new browseAllSelect: [:m| m primitive = 19]"
	primitiveIndex = 19 ifTrue:
		[ToolSet 
			debugContext: self
			label:'Code simulation error'
			contents: nil].

	"ContextPart>>blockCopy:; simulated to get startpc right"
	(primitiveIndex = 80 and: [(self objectClass: receiver) includesBehavior: ContextPart]) 
		ifTrue: [^self push: ((BlockContext newForMethod: receiver method)
						home: receiver home
						startpc: pc + 2
						nargs: (arguments at: 1))].
	(primitiveIndex = 81 and: [(self objectClass: receiver) == BlockContext]) "BlockContext>>value[:value:...]"
		ifTrue: [^receiver pushArgs: arguments from: self].
	(primitiveIndex = 82 and: [(self objectClass: receiver) == BlockContext]) "BlockContext>>valueWithArguments:"
		ifTrue: [^receiver pushArgs: arguments first from: self].
	primitiveIndex = 83 "afr 9/11/1998 19:50" "Object>>perform:[with:...]"
		ifTrue: [^self send: arguments first
					to: receiver
					with: arguments allButFirst
					super: false].
	primitiveIndex = 84 "afr 9/11/1998 19:50 & eem 8/18/2009 17:04" "Object>>perform:withArguments:"
		ifTrue: [^self send: arguments first
					to: receiver
					with: (arguments at: 2)
					startClass: nil].
	primitiveIndex = 100 "eem 8/18/2009 16:57" "Object>>perform:withArguments:inSuperclass:"
		ifTrue: [^self send: arguments first
					to: receiver
					with: (arguments at: 2)
					startClass: (arguments at: 3)].

	"Mutex>>primitiveEnterCriticalSection
	 Mutex>>primitiveTestAndSetOwnershipOfCriticalSection"
	(primitiveIndex = 186 or: [primitiveIndex = 187]) ifTrue:
		[| active effective |
		 active := Processor activeProcess.
		 effective := active effectiveProcess.
		 "active == effective"
		 value := primitiveIndex = 186
					ifTrue: [receiver primitiveEnterCriticalSectionOnBehalfOf: effective]
					ifFalse: [receiver primitiveTestAndSetOwnershipOfCriticalSectionOnBehalfOf: effective].
		 ^((self objectClass: value) == Array
		    and: [value size = 2
		    and: [value first == PrimitiveFailToken]])
			ifTrue: [value]
			ifFalse: [self push: value]].

	primitiveIndex = 188 ifTrue: "eem 5/27/2008 11:10 Object>>withArgs:executeMethod:"
		[^MethodContext
			sender: self
			receiver: receiver
			method: (arguments at: 2)
			arguments: (arguments at: 1)].

	"Closure primitives"
	(primitiveIndex = 200 and: [self == receiver]) ifTrue:
		"ContextPart>>closureCopy:copiedValues:; simulated to get startpc right"
		[^self push: (BlockClosure
						outerContext: receiver
						startpc: pc + 2
						numArgs: arguments first
						copiedValues: arguments last)].
	((primitiveIndex between: 201 and: 205)			 "BlockClosure>>value[:value:...]"
	or: [primitiveIndex between: 221 and: 222]) ifTrue: "BlockClosure>>valueNoContextSwitch[:]"
		[^receiver simulateValueWithArguments: arguments caller: self].
	primitiveIndex = 206 ifTrue:						"BlockClosure>>valueWithArguments:"
		[^receiver simulateValueWithArguments: arguments first caller: self].

	primitiveIndex = 118 ifTrue: "tryPrimitive:withArgs:; avoid recursing in the VM"
		[(arguments size = 2
		 and: [arguments first isInteger
		 and: [(self objectClass: arguments last) == Array]]) ifFalse:
			[^ContextPart primitiveFailTokenFor: nil].
		 ^self doPrimitive: arguments first method: meth receiver: receiver args: arguments last].

	value := primitiveIndex = 120 "FFI method"
				ifTrue: [(meth literalAt: 1) tryInvokeWithArguments: arguments]
				ifFalse:
					[primitiveIndex = 117 "named primitives"
						ifTrue: [self tryNamedPrimitiveIn: meth for: receiver withArgs: arguments]
						ifFalse:
							[receiver tryPrimitive: primitiveIndex withArgs: arguments]].
	^((self objectClass: value) == Array
	    and: [value size = 2
	    and: [value first == PrimitiveFailToken]])
		ifTrue: [value]
		ifFalse: [self push: (self valueOrConstraintVariableFor: meth receiver: receiver value: value)]! !

!ContextPart methodsFor: '*Constraints' stamp: 'mg 4/23/2014 17:08'!
send: selector super: superFlag numArgs: numArgs
	"Simulate the action of bytecodes that send a message with selector, 
	selector. The argument, superFlag, tells whether the receiver of the 
	message was specified with 'super' in the source method. The arguments 
	of the message are found in the top numArgs locations on the stack and 
	the receiver just below them."
	<layer: #ccm>
	| receiver arguments returnValue pair error |
	arguments := Array new: numArgs.
	numArgs to: 1 by: -1 do: [ :i | arguments at: i put: self pop].
	receiver := self pop.
	(receiver class category =  #'Constraints-Core' or: [receiver class category =  #'Cassowary-Kernel'])
		ifTrue: ["If we are inside the constraint solver, step to the outer context."
				(Array with: #ccm) deactivateLayersInCurrentProcess.
				QuickStep == self 
					ifTrue: [QuickStep := nil.
						returnValue := self quickSend: selector to: receiver with: arguments super: superFlag]
					ifFalse: [returnValue := self send: selector to: receiver with: arguments super: superFlag].
				pair := returnValue runUntilErrorOrReturnFrom: returnValue.	
				error := pair second.
				error ifNotNil: [Transcript show: 'an error occured!!'; cr. ^ error signalerContext].
				(Array with: #ccm) activateLayersInCurrentProcess.
				^ pair first].
	selector = 'terminateProcess' 
		ifTrue: [Transcript show: 'process is terminating'; cr. ^ nil].
	QuickStep == self ifTrue:
		[QuickStep := nil.
		^self quickSend: selector to: receiver with: arguments super: superFlag].
	^self send: selector to: receiver with: arguments super: superFlag! !

!ContextPart methodsFor: '*Constraints' stamp: 'mg 4/23/2014 17:13'!
valueOrConstraintVariableFor: method receiver: receiver value: value
	<layer: #ccm>
	| setter cv |
	method isMwMethodWrapper
		ifTrue: [	"replace the corresponding instance variable with a constraint variable object."
			Transcript show: 'found already wrapped instVar '; show: method selector; cr.
			cv := value asConstraintVariable.
			setter := method methodClass methodDict at: (method selector asString, ':') asSymbol.
			setter isMwMethodWrapper
				ifFalse: [self error: 'The getter is wrapped but the setter is not wrapped!!'].
			setter unwrappedMethod valueWithReceiver: receiver arguments: (Array with: cv).
			^ cv].
	method isReturnField 
		ifTrue: [	"replace the corresponding instance variable with a constraint variable object."
			Transcript show: 'found instVar '; show: method selector; cr.
			cv := value asConstraintVariable.
			setter := method methodClass methodDict at: (method selector asString, ':') asSymbol.
			setter unwrappedMethod valueWithReceiver: receiver arguments: (Array with: cv).
			"install method wrappers for the accessors"
			(ConstraintMethodWrapperForGetter on: method selector inClass: method methodClass) 
				install.
			(ConstraintMethodWrapperForSetter on: setter selector inClass: method methodClass)  
				getMethod: method unwrappedMethod;
				install.
			^ cv].
	^ value! !

!BlockClosure methodsFor: '*Constraints' stamp: 'mg 4/15/2014 13:45'!
alwaysTrue
	"This method constructs constraints for all solvable constraints inside this blockClosure."
	| newProcess |
	newProcess := self newTerminatingProcess.
	#ccm withLayerDo: [[newProcess step] doWhileFalse: [newProcess suspendedContext isNil]].
	newProcess terminate.! !

!BlockClosure methodsFor: '*Constraints' stamp: 'mg 4/15/2014 13:33'!
newTerminatingProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."
	<primitive: 19> "Simulation guard"
	^Process
		forContext: 
			[self value. self terminateProcess.] asContext
		priority: Processor activePriority! !

!BlockClosure methodsFor: '*Constraints' stamp: 'mg 4/15/2014 13:33'!
terminateProcess

	^ self! !

!ClStrength methodsFor: '*Constraints' stamp: 'mg 4/16/2014 15:32'!
isMedium

	^ self == MediumStrength! !

!ClStrength methodsFor: '*Constraints' stamp: 'mg 4/16/2014 15:32'!
isStrong

	^ self == StrongStrength! !

!ClStrength methodsFor: '*Constraints' stamp: 'mg 4/16/2014 15:32'!
isWeak

	^ self == WeakStrength! !

MwBlockMethodWrapper subclass: #ConstraintMethodWrapperForGetter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintMethodWrapperForGetter methodsFor: 'evaluating' stamp: 'mg 4/22/2014 14:27'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	| returnValue |
	returnValue := self clientMethod valueWithReceiver: anObject arguments: anArrayOfObjects.
	^ returnValue value! !

MwBlockMethodWrapper subclass: #ConstraintMethodWrapperForSetter
	instanceVariableNames: 'getMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintMethodWrapperForSetter methodsFor: 'accessing' stamp: 'mg 4/22/2014 14:34'!
getMethod

	^ getMethod! !

!ConstraintMethodWrapperForSetter methodsFor: 'accessing' stamp: 'mg 4/22/2014 14:34'!
getMethod: anObject

	getMethod := anObject! !

!ConstraintMethodWrapperForSetter methodsFor: 'evaluating' stamp: 'mg 4/22/2014 15:17'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	| constraintVariable |
	constraintVariable := self getMethod valueWithReceiver: anObject arguments: Array new.
	constraintVariable suggestValue: anArrayOfObjects first.! !

Error subclass: #ConstraintUnsatisfiableError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

Object subclass: #Constraint
	instanceVariableNames: 'expression solver priority externalConstraint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!Constraint methodsFor: 'comparing' stamp: 'mg 4/16/2014 12:02'!
< aConstraint

	self priority < aConstraint priority! !

!Constraint methodsFor: 'comparing' stamp: 'mg 4/16/2014 11:46'!
<= aConstraint

	self priority <= aConstraint priority! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/23/2014 10:38'!
expression

	^ expression! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/23/2014 10:38'!
expression: anObject

	expression := anObject! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
externalConstraint

	^ externalConstraint! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
externalConstraint: anObject

	externalConstraint := anObject! !

!Constraint methodsFor: 'initialize-release' stamp: 'mg 4/16/2014 11:58'!
initialize

	priority := ConstraintPriority required.! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
priority

	^ priority! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
priority: anObject

	priority := anObject! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:12'!
solver

	^ solver! !

!Constraint methodsFor: 'accessing' stamp: 'mg 4/16/2014 16:33'!
solver: anObject

	solver := anObject.
	anObject addConstraint: self.! !

!Constraint methodsFor: 'as yet unclassified' stamp: 'mg 4/23/2014 14:01'!
triggerSolver
	"This method should be called, if this constraint changed, e.g., because a variable changed. It triggers the solver to solve all constraints. The solving process may raise an exception, if the constraints are unsatisfiable."
	self solver solve.
! !

Object subclass: #ConstraintGenericVariable
	instanceVariableNames: 'constraint originalValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintGenericVariable commentStamp: 'mg 4/16/2014 13:02' prior: 0!
A ConstraintGenericVariable represents all values that are used inside a constraint but does not have a concrete ConstraintVariable.

Instance Variables
	constraint:		The constraint is needed to invalidate and recreate the constraint, if this generic variable changes its value.
	originalValue:	The original object
!

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 15:56'!
asConstraintVariable

	^ self! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
constraint

	^ constraint! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
constraint: anObject

	constraint := anObject! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:42'!
isCassowaryVariable

	^ false! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:49'!
isConstant
	"In general, constraint variables can be changed."
	^ false! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
originalValue

	^ originalValue! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 12:52'!
originalValue: anObject

	originalValue := anObject! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:26'!
suggestValue: aValue
	"This method adds an equality constraint for the new value. If all constraints are still solvable, the value is changed to aValue. Otherwise, the old value does not change and an exception is raised. Generic variables are not supposed to be changed. If so, all corresponding constraints should be recreated."! !

!ConstraintGenericVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:06'!
value
	"Returns the value from the solver variable. A generic variable forwards all messages to the original object."
	^ originalValue! !

ConstraintGenericVariable subclass: #ConstraintVariable
	instanceVariableNames: 'externalVariable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintVariable commentStamp: 'mg 4/16/2014 13:07' prior: 0!
A ConstraintVariable is a concrete ConstraintVariable that does have a solver object for it.

Instance Variables
	externalVariable:	This is the solver object for the constraint variable.
!

ConstraintVariable subclass: #ConstraintCassowaryVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintCassowaryVariable methodsFor: 'comparing' stamp: 'mg 4/24/2014 10:55'!
* x
	"This method should return a linear expression. x should be a number or constant. Otherwise, Cassowary raises an error."
	^ self asConstraintLinearExpression * x asConstantConstraintVariable value! !

!ConstraintCassowaryVariable methodsFor: 'comparing' stamp: 'mg 4/23/2014 11:19'!
+ x
	"This method should return a linear expression"
	^ self asConstraintLinearExpression + x asConstraintLinearExpression! !

!ConstraintCassowaryVariable methodsFor: 'comparing' stamp: 'mg 4/23/2014 14:07'!
- x
	"This method should return a linear expression"
	^ self asConstraintLinearExpression - x asConstraintLinearExpression! !

!ConstraintCassowaryVariable methodsFor: 'comparing' stamp: 'mg 4/23/2014 13:53'!
= x

	^ self asConstraintLinearExpression = x asConstraintLinearExpression! !

!ConstraintCassowaryVariable methodsFor: 'accessing' stamp: 'mg 4/24/2014 10:56'!
asConstantConstraintVariable

	self error: 'A Cassowary variable is not constant.'.! !

!ConstraintCassowaryVariable methodsFor: 'accessing' stamp: 'mg 4/23/2014 11:00'!
asConstraintLinearExpression

	| expr |
	expr := ConstraintLinearExpression new.
	expr externalExpression: self externalVariable asLinearExpression.
	^ expr! !

!ConstraintCassowaryVariable methodsFor: 'initialize-release' stamp: 'mg 4/17/2014 13:37'!
initialize

	externalVariable := ClVariable newWith: self originalValue.! !

!ConstraintCassowaryVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 14:42'!
isCassowaryVariable
	
	^ true! !

!ConstraintCassowaryVariable methodsFor: 'accessing' stamp: 'mg 4/23/2014 10:26'!
suggestValue: aValue

	| extConstraint |
	extConstraint := self externalVariable cnEqual: aValue.
	[constraint := Constraint new 
		externalConstraint: extConstraint;
		priority: (ConstraintPriority fromCassowaryStrength: extConstraint strength);
		solver: ConstraintSolver cassowarySolver.
	constraint solver addStay: self.
	constraint triggerSolver] ifError: [ConstraintUnsatisfiableError new 
			signal: 'The value cannot be assigned because the constraints are unsatisfiable.'].
	ConstraintSolver cassowarySolver removeConstraint: constraint.! !

ConstraintVariable subclass: #ConstraintConstantVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintConstantVariable methodsFor: 'comparing' stamp: 'mg 4/23/2014 13:57'!
= x

	^ self asConstraintLinearExpression = x asConstraintLinearExpression! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 11:00'!
asConstantConstraintVariable

	^ self! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/23/2014 13:55'!
asConstraintLinearExpression

	^ self value asConstraintLinearExpression! !

!ConstraintConstantVariable methodsFor: 'initialize-release' stamp: 'mg 4/17/2014 11:15'!
initialize
	"The value of a constant variable should never be nil, because it cannot be changed afterwards."

	originalValue := -1.! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:47'!
isConstant
	"Constant variables are read-only."
	^ true! !

!ConstraintConstantVariable methodsFor: 'accessing' stamp: 'mg 4/22/2014 11:38'!
suggestValue: aValue

	self error: 'The constraint variable is read-only.'.! !

!ConstraintVariable class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 14:37'!
newCassowaryVariableFor: aNumber

	| cv extVar |
	cv := ConstraintCassowaryVariable new.
	cv originalValue: aNumber.
	extVar := ClVariable newWith: aNumber asFloat name: aNumber name.
	cv externalVariable: extVar.
	^ cv! !

!ConstraintVariable class methodsFor: 'instance creation' stamp: 'mg 4/17/2014 10:59'!
newConstantVariableFor: aNumber

	| cv extVar |
	cv := ConstraintConstantVariable new.
	cv originalValue: aNumber.
	"maybe the external variable is another one?"
	extVar := ClVariable newWith: aNumber asFloat name: aNumber name.
	cv externalVariable: extVar.
	^ cv! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/17/2014 10:54'!
asConstantConstraintVariable

	^ ConstraintConstantVariable new
			constraint: self constraint;
			originalValue: self originalValue;
			externalVariable: self externalVariable! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:21'!
externalVariable

	^ externalVariable! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:21'!
externalVariable: anObject

	externalVariable := anObject! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:19'!
suggestValue: aValue
	"This method adds an equality constraint for the new value. If all constraints are still solvable, the value is changed to aValue. Otherwise, the old value does not change and an exception is raised."! !

!ConstraintVariable methodsFor: 'accessing' stamp: 'mg 4/23/2014 10:14'!
value
	"Returns the value from the solver variable."
	^ externalVariable value! !

Object subclass: #ConstraintLinearExpression
	instanceVariableNames: 'externalExpression'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/24/2014 11:02'!
* x

	^ ConstraintLinearExpression new
		externalExpression: self externalExpression * x asConstantConstraintVariable value! !

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/23/2014 14:47'!
+ x

	| expr |
	expr := x asConstraintLinearExpression.
	^ self addExpression: expr! !

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/23/2014 14:47'!
- x

	| expr |
	expr := x asConstraintLinearExpression.
	^ self subtractExpression: expr! !

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/23/2014 14:47'!
= x

	| expr |
	expr := x asConstraintLinearExpression.
	^ self equalsExpression: expr! !

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/23/2014 10:55'!
addExpression: aConstraintLinearExpression

	^ ConstraintLinearExpression new
		externalExpression: self externalExpression + aConstraintLinearExpression externalExpression! !

!ConstraintLinearExpression methodsFor: 'accessing' stamp: 'mg 4/23/2014 11:01'!
asConstraintLinearExpression

	^ self! !

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/23/2014 14:00'!
equalsExpression: aConstraintLinearExpression

	| extConstraint constraint |
	extConstraint := self externalExpression cnEqual: aConstraintLinearExpression externalExpression.
	constraint := Constraint new 
		expression: self;
		externalConstraint: extConstraint;
		priority: (ConstraintPriority fromCassowaryStrength: extConstraint strength);
		solver: ConstraintSolver cassowarySolver.
	constraint triggerSolver.
	^ constraint
	! !

!ConstraintLinearExpression methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:20'!
externalExpression

	^ externalExpression! !

!ConstraintLinearExpression methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:20'!
externalExpression: anObject

	externalExpression := anObject! !

!ConstraintLinearExpression methodsFor: 'comparing' stamp: 'mg 4/23/2014 14:07'!
subtractExpression: aConstraintLinearExpression

	^ ConstraintLinearExpression new
		externalExpression: self externalExpression - aConstraintLinearExpression externalExpression! !

Object subclass: #ConstraintPriority
	instanceVariableNames: 'name val'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

!ConstraintPriority commentStamp: 'mg 4/16/2014 12:05' prior: 0!
A ConstraintPriority is a named priority object. It represents the priority of a constraint.

Instance Variables
	name:	symbolic name for a priority
	val:		a number for the priority to simplify comparisons between priorities 

!

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 15:34'!
fromCassowaryStrength: aClStrength

	aClStrength isRequired ifTrue: [^ ConstraintPriority required].
	aClStrength isStrong ifTrue: [^ ConstraintPriority strong].
	aClStrength isMedium ifTrue: [^ ConstraintPriority medium].
	aClStrength isWeak ifTrue: [^ ConstraintPriority weak].! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:56'!
medium

	^ self new 
		name: 'medium';
		val: 2! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:55'!
required

	^ self new 
		name: 'required';
		val: 0! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:55'!
strong

	^ self new 
		name: 'strong';
		val: 1! !

!ConstraintPriority class methodsFor: 'instance creation' stamp: 'mg 4/16/2014 11:56'!
weak

	^ self new 
		name: 'weak';
		val: 3! !

!ConstraintPriority methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 12:02'!
< aConstraintPriority

	^ self val < aConstraintPriority val! !

!ConstraintPriority methodsFor: 'as yet unclassified' stamp: 'mg 4/16/2014 11:58'!
<= aConstraintPriority

	^ self val <= aConstraintPriority val! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:57'!
name
		
	^ name! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:56'!
name: anObject

	name := anObject! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:56'!
val

	^ val! !

!ConstraintPriority methodsFor: 'accessing' stamp: 'mg 4/16/2014 11:56'!
val: anObject

	val := anObject! !

Object subclass: #ConstraintSolver
	instanceVariableNames: 'externalSolver constraints stayConstraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Core'!

ConstraintSolver class
	instanceVariableNames: 'CassowarySolver ExternalCassowarySolver'!

!ConstraintSolver commentStamp: 'mg 4/23/2014 10:30' prior: 0!
A ConstraintSolver is xxxxxxxxx.

Instance Variables
	constraints:		<Object>
	externalSolver:		<Object>
	stayConstraints:	This collection consists of ConstraintVariables, that have a weak stay constraint.

constraints
	- xxxxx

externalSolver
	- xxxxx

stayConstraints
	- xxxxx
!

ConstraintSolver class
	instanceVariableNames: 'CassowarySolver ExternalCassowarySolver'!

!ConstraintSolver class methodsFor: 'instance creation' stamp: 'mg 4/22/2014 15:19'!
cassowarySolver

	CassowarySolver ifNil: [CassowarySolver := self basicNew initialize].
	ExternalCassowarySolver ifNil: [ExternalCassowarySolver := ClSimplexSolver new].
	CassowarySolver externalSolver: ExternalCassowarySolver.
	^ CassowarySolver! !

!ConstraintSolver methodsFor: 'manipulating' stamp: 'mg 4/17/2014 13:40'!
addConstraint: aConstraint

	self constraints add: aConstraint.
	self externalSolver addConstraint: aConstraint externalConstraint.! !

!ConstraintSolver methodsFor: 'manipulating' stamp: 'mg 4/23/2014 10:29'!
addStay: aConstraintVariable

	self stayConstraints add: aConstraintVariable.
	self externalSolver addStay: aConstraintVariable externalVariable.! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
constraints

	^ constraints! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
constraints: anObject

	constraints := anObject! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
externalSolver

	^ externalSolver! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/16/2014 13:11'!
externalSolver: anObject

	externalSolver := anObject! !

!ConstraintSolver methodsFor: 'initialize-release' stamp: 'mg 4/23/2014 10:28'!
initialize

	externalSolver := ClSimplexSolver new.
	constraints := OrderedCollection new.
	stayConstraints := OrderedCollection new.! !

!ConstraintSolver methodsFor: 'manipulating' stamp: 'mg 4/23/2014 10:33'!
removeAllConstraints

	self constraints removeAll.
	self stayConstraints removeAll.
	self externalSolver initialize.! !

!ConstraintSolver methodsFor: 'manipulating' stamp: 'mg 4/17/2014 13:40'!
removeConstraint: aConstraint

	self constraints remove: aConstraint.
	self externalSolver removeConstraint: aConstraint externalConstraint.! !

!ConstraintSolver methodsFor: 'manipulating' stamp: 'mg 4/16/2014 13:20'!
solve
	"This method triggers the external solver to solve the constraints. It may raise an exception, if the constraints are unsatisfiable."
	self externalSolver solve.! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/23/2014 10:28'!
stayConstraints

	^ stayConstraints! !

!ConstraintSolver methodsFor: 'accessing' stamp: 'mg 4/23/2014 10:28'!
stayConstraints: anObject

	stayConstraints := anObject! !

Object subclass: #ExampleClass
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
a

	^ a! !

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
a: anObject

	a := anObject! !

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
b

	^ b! !

!ExampleClass methodsFor: 'accessing' stamp: 'mg 4/15/2014 14:31'!
b: anObject

	b := anObject! !

!ExampleClass methodsFor: 'initialize-release' stamp: 'mg 4/15/2014 14:33'!
initialize

	a := 1.
	b := 2.! !

!Object methodsFor: '*Constraints' stamp: 'mg 4/17/2014 11:02'!
asConstantConstraintVariable

	self error: 'This object cannot be converted to a constant constraint variable because it has no external variable.'.! !

!Object methodsFor: '*Constraints' stamp: 'mg 4/16/2014 13:38'!
asConstraintVariable

	^ ConstraintGenericVariable new originalValue: self! !

!ClVariable methodsFor: '*Constraints' stamp: 'mg 4/17/2014 16:37'!
asConstantConstraintVariable

	^ self! !

!ClVariable methodsFor: '*Constraints' stamp: 'mg 4/17/2014 16:36'!
asConstraintVariable

	^ self! !

TestCase subclass: #ConstraintsTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!ConstraintsTests methodsFor: 'running' stamp: 'mg 4/23/2014 10:33'!
setUp

	| wrappers |
	ConstraintSolver cassowarySolver removeAllConstraints.

	[wrappers := ExampleClass methodDict values select: [:method | method isMwMethodWrapper]. 
	wrappers do: [:w | w uninstall]] doWhileFalse: [wrappers isEmpty].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 09:59'!
testChangeOfSingleVariableValue

	| ex |
	ex := ExampleClass new.
	self shouldnt: [[ex a = 42] alwaysTrue.] raise: Error.
	self should: [ex a = 42].

	self should: [ex a: 23] raise: ConstraintUnsatisfiableError.
	self should: [ex a = 42].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 17:06'!
testComplexLinearExpression

	| ex ex2 ex3 |
	ex := ExampleClass new.
	ex a: 23.
	ex b: 24.
	ex2 := ExampleClass new.
	ex2 a: 17.
	ex2 b: 18.
	ex3 := ExampleClass new.
	ex3 a: 41.
	ex3 b: 42.

	self shouldnt: [[(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))] alwaysTrue] raise: Error.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].

	self shouldnt: [ex a: 1] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].
	self should: [ex a = 1].

	self shouldnt: [ex2 b: 2] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].
	self should: [ex2 b = 2].

	self shouldnt: [ex3 a: 3] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].
	self should: [ex3 a = 3].

	self shouldnt: [ex2 a: 4] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].
	self should: [ex2 a = 4].

	self shouldnt: [ex b: 5] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].
	self should: [ex b = 5].

	self shouldnt: [ex3 b: 6] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].
	self should: [ex3 b = 6].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/24/2014 11:15'!
testComplexLinearExpressionWithMultiplication

	| ex ex2 ex3 |
	ex := ExampleClass new.
	ex a: 23.
	ex b: 24.
	ex2 := ExampleClass new.
	ex2 a: 17.
	ex2 b: 18.
	ex3 := ExampleClass new.
	ex3 a: 41.
	ex3 b: 42.

	self shouldnt: [[(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))] alwaysTrue] raise: Error.
	self should: [(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))].

	self shouldnt: [ex a: 1] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))].
	self should: [ex a = 1].

	self shouldnt: [ex2 b: 2] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))].
	self should: [ex2 b = 2].

	self shouldnt: [ex2 a: 4] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))].
	self should: [ex2 a = 4].

	self shouldnt: [ex b: 5] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))].
	self should: [ex b = 5].

	self shouldnt: [ex3 a: 6] raise: ConstraintUnsatisfiableError.
	self should: [(ex a - ex2 b) * 3 = (ex2 a * 2 + (ex b - ex3 a * (-4)))].
	self should: [ex3 a = 6].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 15:03'!
testEqualityOfTwoVariables

	| ex ex2 |
	ex := ExampleClass new.
	ex a: 23.
	ex2 := ExampleClass new.
	ex2 a: 17.

	self shouldnt: [[ex a = ex2 a] alwaysTrue] raise: Error.
	self should: [ex a = ex2 a].

	self shouldnt: [ex2 a: 42] raise: ConstraintUnsatisfiableError.
	self should: [ex a = ex2 a].
	self should: [ex2 a = 42].

	self shouldnt: [ex a: 9] raise: ConstraintUnsatisfiableError.
	self should: [ex a = ex2 a].
	self should: [ex a = 9].

	self shouldnt: [ex2 a: 21] raise: ConstraintUnsatisfiableError.
	self should: [ex a = ex2 a].
	self should: [ex2 a = 21].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 17:05'!
testInstancevariableClass

	| ex ex2 ex3 |
	ex := ExampleClass new.
	ex a: 23.
	ex b: 24.
	ex2 := ExampleClass new.
	ex2 a: 17.
	ex2 b: 18.
	ex3 := ExampleClass new.
	ex3 a: 41.
	ex3 b: 42.

	self shouldnt: [[(ex a - ex2 b) + ex3 a = (ex b + ex3 b)] alwaysTrue] raise: Error.

	self should: [(ex instVarNamed: 'a') class == ConstraintCassowaryVariable].
	self should: [(ex instVarNamed: 'b') class == ConstraintCassowaryVariable].

	self shouldnt: [(ex2 instVarNamed: 'a') class == ConstraintCassowaryVariable].
	self should: [(ex2 instVarNamed: 'b') class == ConstraintCassowaryVariable].

	self should: [(ex3 instVarNamed: 'a') class == ConstraintCassowaryVariable].
	self should: [(ex3 instVarNamed: 'b') class == ConstraintCassowaryVariable].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:19'!
testLinearExpressionAddition

	| ex ex2 |
	ex := ExampleClass new.
	ex a: 23.
	ex2 := ExampleClass new.
	ex2 a: 17.

	self shouldnt: [[ex a + ex2 a = 42] alwaysTrue] raise: Error.
	self should: [ex a + ex2 a = 42].

	self shouldnt: [ex a: 13] raise: ConstraintUnsatisfiableError.
	self should: [ex a + ex2 a = 42].
	self should: [ex a = 13].

	self shouldnt: [ex2 a: 17] raise: ConstraintUnsatisfiableError.
	self should: [ex a + ex2 a = 42].
	self should: [ex2 a = 17].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/24/2014 11:07'!
testLinearExpressionMultiplication

	| ex ex2 |
	ex := ExampleClass new.
	ex a: 23.
	ex2 := ExampleClass new.
	ex2 a: 17.

	self shouldnt: [[ex a * 3 = 42] alwaysTrue] raise: Error.
	self should: [ex a * 3 = 42].

	self should: [ex a: 13] raise: ConstraintUnsatisfiableError.
	self should: [ex a * 3 = 42].
	self should: [ex a = 14].

	self shouldnt: [ex a: 14] raise: ConstraintUnsatisfiableError.
	self should: [ex a * 3 = 42].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:29'!
testLinearExpressionSubtraction

	| ex ex2 |
	ex := ExampleClass new.
	ex a: 23.
	ex2 := ExampleClass new.
	ex2 a: 17.

	self shouldnt: [[ex a - ex2 a = 42] alwaysTrue] raise: Error.
	self should: [ex a - ex2 a = 42].

	self shouldnt: [ex a: 13] raise: ConstraintUnsatisfiableError.
	self should: [ex a - ex2 a = 42].
	self should: [ex a = 13].

	self shouldnt: [ex2 a: 17] raise: ConstraintUnsatisfiableError.
	self should: [ex a - ex2 a = 42].
	self should: [ex2 a = 17].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/23/2014 15:06'!
testMethodWrapper

	| ex ex2 ex3 |
	ex := ExampleClass new.
	ex a: 23.
	ex b: 24.
	ex2 := ExampleClass new.
	ex2 a: 17.
	ex2 b: 18.
	ex3 := ExampleClass new.
	ex3 a: 41.
	ex3 b: 42.

	self shouldnt: [[(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))] alwaysTrue] raise: Error.
	self should: [(ex a - ex2 b) + ex3 a = (ex2 a - (ex b + ex3 b))].

	self should: [(ExampleClass methodDict at: #a) class == ConstraintMethodWrapperForGetter].
	self should: [(ExampleClass methodDict at: #b) class == ConstraintMethodWrapperForGetter].

	self should: [(ExampleClass methodDict at: #a:) class == ConstraintMethodWrapperForSetter].
	self should: [(ExampleClass methodDict at: #b:) class == ConstraintMethodWrapperForSetter].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/22/2014 11:10'!
testNoException

	self should: [[1 = 1] alwaysTrue].
	self shouldnt: [[1 = 1] alwaysTrue] raise: ConstraintUnsatisfiableError.! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/16/2014 12:02'!
testPriorityOrder

	| p1 p2 p3 p4 |
	p1 := ConstraintPriority required.
	p2 := ConstraintPriority strong.
	p3 := ConstraintPriority medium.
	p4 := ConstraintPriority weak.

	self should: [p1 <= p2].
	self should: [p2 <= p3].
	self should: [p3 <= p4].

	self should: [p1 < p2].
	self should: [p2 < p3].
	self should: [p3 < p4].

	self shouldnt: [p1 < p1].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/22/2014 15:50'!
testSimpleAssignmentConstraint

	| ex |
	ex := ExampleClass new.
	self shouldnt: [[ex a = 42] alwaysTrue.] raise: Error.
	self should: [ex a isNumber].
	self should: [ex a = 42].! !

!ConstraintsTests methodsFor: 'testing' stamp: 'mg 4/22/2014 12:02'!
testUnsatisfiableConstraints

	self shouldnt: [[1 = 2] alwaysTrue].! !

TestCase subclass: #LinearExpressionTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!LinearExpressionTests methodsFor: 'running' stamp: 'mg 4/23/2014 11:08'!
setUp

	ConstraintSolver cassowarySolver removeAllConstraints.! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:48'!
testAddition

	| var17 var23 expr expr2 |
	var17 := 17 asConstraintVariable.
	var23 := 23 asConstraintVariable.

	self should: [(var17 + var23) class == ConstraintLinearExpression].

	expr := 9 asConstraintVariable + 13 asConstraintVariable.
	self should: [(expr + var17) class == ConstraintLinearExpression].
	self should: [(var17 + expr) class == ConstraintLinearExpression].

	expr2 := 42 asConstraintVariable + 21 asConstraintVariable.
	self should: [(expr + expr2) class == ConstraintLinearExpression].

	self should: [(var17 + 2) class == ConstraintLinearExpression].

	self shouldnt: [var23 + var17 = 42] raise: Error.
	self should: [var23 value + var17 value = 42].! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:16'!
testEqualityExpressionsWithAddition

	| var13 var17 var9 var23 |
	var17 := 17 asConstraintVariable.
	var9 := 9 asConstraintVariable.
	var13 := 13 asConstraintVariable.
	var23 := 23 asConstraintVariable.

	self shouldnt: [(var17 + var9) = (var13 + var23)] raise: Error.
	self should: [(var17 value + var9 value) = (var13 value + var23 value)].

	! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:16'!
testEqualityExpressionsWithSubtraction

	| var13 var17 var9 var23 |
	var17 := 17 asConstraintVariable.
	var9 := 9 asConstraintVariable.
	var13 := 13 asConstraintVariable.
	var23 := 23 asConstraintVariable.

	self shouldnt: [(var17 - var9) = (var23 - var13)] raise: Error.
	self should: [(var17 value - var9 value) = (var23 value - var13 value)].

	! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:17'!
testEqualityVariableAndExpressionWithAddition

	| var13 var17 var21 var42 var9 var23 |

	var17 := 17 asConstraintVariable.
	var9 := 9 asConstraintVariable.
	var13 := 13 asConstraintVariable.
	self shouldnt: [var9 + var13 = var17] raise: Error.
	self should: [var9 value + var13 value = var17 value].
	
	var23 := 23 asConstraintVariable.
	var21 := 21 asConstraintVariable.
	var42 := 42 asConstraintVariable.
	self shouldnt: [var42 = (var21 + var23)] raise: Error.
	self should: [var42 value = (var21 value + var23 value)].

	! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:18'!
testEqualityVariableAndExpressionWithSubtraction

	| var13 var17 var21 var42 var9 var23 |

	var17 := 17 asConstraintVariable.
	var9 := 9 asConstraintVariable.
	var13 := 13 asConstraintVariable.
	self shouldnt: [var9 - var13 = var17] raise: Error.
	self should: [var9 value - var13 value = var17 value].
	
	var23 := 23 asConstraintVariable.
	var21 := 21 asConstraintVariable.
	var42 := 42 asConstraintVariable.
	self shouldnt: [var42 = (var21 - var23)] raise: Error.
	self should: [var42 value = (var21 value - var23 value)].

	! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/24/2014 11:03'!
testMultiplication

	| var17 var23 expr expr2 var21 |
	var17 := 17 asConstraintVariable.
	var21 := 21 asConstraintVariable.
	var23 := 23 asConstantConstraintVariable.

	self should: [(var17 * var23) class == ConstraintLinearExpression].
	self should: [(var17 * 3) class == ConstraintLinearExpression].

	expr := 9 asConstraintVariable * 13 asConstantConstraintVariable.
	self should: [(expr * var23) class == ConstraintLinearExpression].
	self should: [(expr * 3) class == ConstraintLinearExpression].

	expr2 := 42 asConstraintVariable * 3.
	self should: [(expr2 * var23) class == ConstraintLinearExpression].
	self should: [(expr2 * 5) class == ConstraintLinearExpression].

	self should: [expr * expr2] raise: Error.
	self should: [var17 * var21] raise: Error.
	self should: [expr * var21] raise: Error.
	self should: [var17 * expr2] raise: Error.

	self shouldnt: [var17 * 3 = 42] raise: Error.
	self should: [var17 value * 3 = 42].! !

!LinearExpressionTests methodsFor: 'testing' stamp: 'mg 4/23/2014 14:39'!
testSubtraction

	| var17 var23 expr expr2 |
	var17 := 17 asConstraintVariable.
	var23 := 23 asConstraintVariable.

	self should: [(var17 - var23) class == ConstraintLinearExpression].

	expr := 9 asConstraintVariable + 13 asConstraintVariable.
	self should: [(expr - var17) class == ConstraintLinearExpression].
	self should: [(var17 - expr) class == ConstraintLinearExpression].

	expr2 := 42 asConstraintVariable + 21 asConstraintVariable.
	self should: [(expr - expr2) class == ConstraintLinearExpression].

	self should: [(var17 - 2) class == ConstraintLinearExpression].

	self shouldnt: [var23 - var17 = 42] raise: Error.
	self should: [var23 value - var17 value = 42].! !

TestCase subclass: #VariableTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Constraints-Tests'!

!VariableTests methodsFor: 'running' stamp: 'mg 4/22/2014 15:27'!
setUp

	ConstraintSolver cassowarySolver removeAllConstraints.! !

!VariableTests methodsFor: 'testing' stamp: 'mg 4/23/2014 13:58'!
testCassowaryVariableEqualityOperatorConstants

	| const2 const5 |
	const2 := 2 asConstantConstraintVariable.
	const5 := 5 asConstantConstraintVariable.

	self should: [const2 isConstant].
	self should: [const5 isConstant].

	self should: [const2 = const5] raise: Error.
	self shouldnt: [const2 = const2 copy] raise: Error.! !

!VariableTests methodsFor: 'testing' stamp: 'mg 4/22/2014 15:37'!
testCassowaryVariableEqualityOperatorNilVariables

	| varNil varNil2 |
	varNil := ConstraintCassowaryVariable new.
	varNil2 := ConstraintCassowaryVariable new.

	self shouldnt: [varNil = varNil2] raise: Error.	"both values are nil"
	self should: [varNil value clApproxZero].
	self should: [varNil2 value clApproxZero].! !

!VariableTests methodsFor: 'testing' stamp: 'mg 4/22/2014 15:38'!
testCassowaryVariableEqualityOperatorOneVariable

	| const2 var17 var23 varNil varTest |
	var17 := ConstraintVariable newCassowaryVariableFor: 17.
	var23 := 23 asConstraintVariable.
	varTest := 'test' asConstraintVariable.
	varNil := ConstraintCassowaryVariable new.
	const2 := 2 asConstantConstraintVariable.

	self should: [var17 isCassowaryVariable].
	self shouldnt: [varTest isCassowaryVariable].

	self shouldnt: [var17 = const2] raise: Error.	"var17 value notNil"
	self should: [var17 value = const2 value].
	self shouldnt: [varNil = const2] raise: Error.	"varNil value isNil"	
	self should: [varNil value = const2 value].
	self shouldnt: [var23 = 47] raise: Error.
	self should: [var23 value = 47].! !

!VariableTests methodsFor: 'testing' stamp: 'mg 4/22/2014 15:38'!
testCassowaryVariableEqualityOperatorTwoVariables

	| var17 var23 varNil varTest |
	var17 := ConstraintVariable newCassowaryVariableFor: 17.
	var23 := 23 asConstraintVariable.
	varTest := 'test' asConstraintVariable.
	varNil := ConstraintCassowaryVariable new.

	self shouldnt: [var17 = var23] raise: Error.	"values different, notNil"
	self should: [var17 value = var23 value].
	self should: [var17 = varTest] raise: Error.	"different variable types"
	self shouldnt: [var23 = varNil] raise: Error. 	"varNil2 value isNil"
	self should: [var23 value = varNil value].! !

!Number methodsFor: '*Constraints' stamp: 'mg 4/17/2014 10:57'!
asConstantConstraintVariable

	^ ConstraintVariable newConstantVariableFor: self! !

!Number methodsFor: '*Constraints' stamp: 'mg 4/23/2014 11:05'!
asConstraintLinearExpression

	^ ConstraintLinearExpression new
		externalExpression: self asLinearExpression! !

!Number methodsFor: '*Constraints' stamp: 'mg 4/16/2014 14:37'!
asConstraintVariable

	^ ConstraintVariable newCassowaryVariableFor: self! !
